package eu.eudat.model.persist;

import eu.eudat.commons.XmlHandlingService;
import eu.eudat.commons.enums.DescriptionStatus;
import eu.eudat.commons.types.descriptiontemplate.DefinitionEntity;
import eu.eudat.commons.validation.BaseValidator;
import eu.eudat.data.DescriptionTemplateEntity;
import eu.eudat.model.DescriptionTemplate;
import gr.cite.tools.exception.MyNotFoundException;
import gr.cite.tools.validation.ValidatorFactory;
import gr.cite.tools.validation.specification.Specification;
import eu.eudat.convention.ConventionService;
import eu.eudat.data.DescriptionEntity;
import eu.eudat.errorcode.ErrorThesaurusProperties;
import eu.eudat.model.persist.descriptionproperties.PropertyDefinitionPersist;
import jakarta.persistence.EntityManager;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.MessageSource;
import org.springframework.context.annotation.Scope;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;
import java.util.UUID;

public class DescriptionPersist {

    private UUID id;

    private String label;

    public static final String _label = "label";

    private UUID dmpId;

    public static final String _dmpId = "dmpId";

    private UUID dmpDescriptionTemplateId;

    public static final String _dmpDescriptionTemplateId = "dmpDescriptionTemplateId";

    private UUID descriptionTemplateId;

    public static final String _descriptionTemplateId = "descriptionTemplateId";

    private DescriptionStatus status;

    public static final String _status = "status";

    private String description;

    private PropertyDefinitionPersist properties;

    public static final String _properties = "properties";

    private List<String> tags;

    public static final String _tags = "tags";


    private String hash;

    public static final String _hash = "hash";

    public UUID getId() {
        return id;
    }

    public void setId(UUID id) {
        this.id = id;
    }

    public String getLabel() {
        return label;
    }

    public void setLabel(String label) {
        this.label = label;
    }

    public UUID getDmpId() {
        return dmpId;
    }

    public void setDmpId(UUID dmpId) {
        this.dmpId = dmpId;
    }

    public UUID getDmpDescriptionTemplateId() {
        return dmpDescriptionTemplateId;
    }

    public void setDmpDescriptionTemplateId(UUID dmpDescriptionTemplateId) {
        this.dmpDescriptionTemplateId = dmpDescriptionTemplateId;
    }

    public DescriptionStatus getStatus() {
        return status;
    }

    public void setStatus(DescriptionStatus status) {
        this.status = status;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public PropertyDefinitionPersist getProperties() {
        return properties;
    }

    public void setProperties(PropertyDefinitionPersist properties) {
        this.properties = properties;
    }

    public List<String> getTags() {
        return tags;
    }

    public void setTags(List<String> tags) {
        this.tags = tags;
    }

    public String getHash() {
        return hash;
    }

    public void setHash(String hash) {
        this.hash = hash;
    }

    public UUID getDescriptionTemplateId() {
        return descriptionTemplateId;
    }

    public void setDescriptionTemplateId(UUID descriptionTemplateId) {
        this.descriptionTemplateId = descriptionTemplateId;
    }

    @Component(DescriptionPersistValidator.ValidatorName)
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public static class DescriptionPersistValidator extends BaseValidator<DescriptionPersist> {

        public static final String ValidatorName = "DescriptionPersistValidator";

        private final MessageSource messageSource;

        private final ValidatorFactory validatorFactory;
        private final EntityManager entityManager;
        private final XmlHandlingService xmlHandlingService;

        protected DescriptionPersistValidator(ConventionService conventionService, ErrorThesaurusProperties errors, MessageSource messageSource, ValidatorFactory validatorFactory, EntityManager entityManager, XmlHandlingService xmlHandlingService) {
            super(conventionService, errors);
            this.messageSource = messageSource;
            this.validatorFactory = validatorFactory;
	        this.entityManager = entityManager;
	        this.xmlHandlingService = xmlHandlingService;
        }

        @Override
        protected Class<DescriptionPersist> modelClass() {
            return DescriptionPersist.class;
        }

        @Override
        protected List<Specification> specifications(DescriptionPersist item) {
            DescriptionTemplateEntity descriptionTemplate  = this.isValidGuid(item.getDescriptionTemplateId()) ? this.entityManager.find(DescriptionTemplateEntity.class, item.getDescriptionTemplateId()) : null;
            DefinitionEntity definition = descriptionTemplate == null ? null : this.xmlHandlingService.fromXmlSafe(DefinitionEntity.class, descriptionTemplate.getDefinition());
            return Arrays.asList(
                    this.spec()
                            .iff(() -> this.isValidGuid(item.getId()))
                            .must(() -> this.isValidHash(item.getHash()))
                            .failOn(DescriptionPersist._hash).failWith(messageSource.getMessage("Validation_Required", new Object[]{DescriptionPersist._hash}, LocaleContextHolder.getLocale())),
                    this.spec()
                            .iff(() -> !this.isValidGuid(item.getId()))
                            .must(() -> !this.isValidHash(item.getHash()))
                            .failOn(DescriptionPersist._hash).failWith(messageSource.getMessage("Validation_OverPosting", new Object[]{}, LocaleContextHolder.getLocale())),
                    this.spec()
                            .must(() -> !this.isEmpty(item.getLabel()))
                            .failOn(DescriptionPersist._label).failWith(messageSource.getMessage("Validation_Required", new Object[]{DescriptionPersist._label}, LocaleContextHolder.getLocale())),
                    this.spec()
                            .iff(() -> !this.isEmpty(item.getLabel()))
                            .must(() -> this.lessEqualLength(item.getLabel(), DescriptionEntity._labelLength))
                            .failOn(DescriptionPersist._label).failWith(messageSource.getMessage("Validation_MaxLength", new Object[]{DescriptionPersist._label}, LocaleContextHolder.getLocale())),
                    this.spec()
                            .must(() -> this.isValidGuid(item.getDmpId()))
                            .failOn(DescriptionPersist._dmpId).failWith(messageSource.getMessage("Validation_Required", new Object[]{DescriptionPersist._dmpId}, LocaleContextHolder.getLocale())),
                    this.spec()
                            .must(() -> this.isValidGuid(item.getDescriptionTemplateId()))
                            .failOn(DescriptionPersist._descriptionTemplateId).failWith(messageSource.getMessage("Validation_Required", new Object[]{DescriptionPersist._descriptionTemplateId}, LocaleContextHolder.getLocale())),
                    this.spec()
                            .must(() -> this.isValidGuid(item.getDmpDescriptionTemplateId()))
                            .failOn(DescriptionPersist._dmpDescriptionTemplateId).failWith(messageSource.getMessage("Validation_Required", new Object[]{DescriptionPersist._dmpDescriptionTemplateId}, LocaleContextHolder.getLocale())),
                    this.spec()
                            .must(() -> !this.isNull(item.getStatus()))
                            .failOn(DescriptionPersist._status).failWith(messageSource.getMessage("Validation_Required", new Object[]{DescriptionPersist._status}, LocaleContextHolder.getLocale())),
                    this.spec()
                            .iff(() -> item.getStatus() == DescriptionStatus.Finalized)
                            .must(() -> !this.isNull(item.getProperties()))
                            .failOn(DescriptionPersist._properties).failWith(messageSource.getMessage("Validation_Required", new Object[]{DescriptionPersist._properties}, LocaleContextHolder.getLocale()))
//                    this.refSpec()
//                            .iff(() -> !this.isNull(item.getProperties()))
//                            .on(DescriptionPersist._properties)
//                            .over(item.getProperties())
//                            .using(() -> this.validatorFactory.validator(PropertyDefinitionPersist.PropertyDefinitionPersistValidator.class).setStatus(item.getStatus()).withDefinition(definition))
//                    this.navSpec()
//                            .iff(() -> !this.isNull(item.getTags()))
//                            .on(DescriptionPersist._tags)
//                            .over(item.getTags())
//                            .using((itm) -> this.validatorFactory.validator(TagPersist.TagPersistValidator.class))
                    );
        }
    }

}
